import uuid
import json
import os

# ######## Default Values ###########

# Define default widget size
DEF_HEIGHT = 700
DEF_WIDTH = 1098 # Same as cell width of Jupyter

HTML_TEMPLATE_FILE = 'template.html'
STYLE_FILE = 'default_style.json'

# Default network
DEF_NODES = [
    {'data': {'id': 'Network Data'}},
    {'data': {'id': 'Empty'}}
]

DEF_EDGES = [
    {'data': {'id': 'is', 'source': 'Network Data', 'target': 'Empty'}}
]

DEF_LAYOUT = 'preset'
DEF_STYLE = 'default'

PRESET_LAYOUTS = {
    'Preset': 'preset',
    'Circle': 'circle',
    'Concentric': 'concentric',
    'Breadthfirst': 'breadthfirst',
    'Spring': 'cose',
    'Grid': 'grid'
}


# Init styles
style_file = open(os.path.abspath(os.path.dirname(__file__)) + '/' + STYLE_FILE, 'r')
style_list = json.load(style_file)
styles = {}
for style in style_list:
    styles[style['title']] = style['style']


def render(network, style=DEF_STYLE, layout_algorithm=DEF_LAYOUT, height=DEF_HEIGHT, width=DEF_WIDTH):
    from jinja2 import Template
    from IPython.core.display import display, HTML

    # Load style file if none available
    if style is None:
        style = styles['default2']
    elif isinstance(style, str):
        # Specified by name
        style = styles[style]

    if network is None:
        nodes = DEF_NODES
        edges = DEF_EDGES
    else:
        nodes = network['elements']['nodes']
        edges = network['elements']['edges']

    path = os.path.abspath(os.path.dirname(__file__)) + '/' + HTML_TEMPLATE_FILE
    template = Template(open(path).read())
    cyjs_widget = template.render(nodes=json.dumps(nodes), edges=json.dumps(edges),
                                  uuid="cy" + str(uuid.uuid4()), widget_width=str(width), widget_height=str(height),
                                  layout=layout_algorithm, style_json=json.dumps(style))

    return display(HTML(cyjs_widget))


# List of available layout algorithms
def get_layouts():
    return PRESET_LAYOUTS


def embed_share(url, width=DEF_WIDTH, height=DEF_HEIGHT):
    from IPython.core.display import display
    from IPython.lib.display import IFrame

    return display(IFrame(url, width, height))


# -*- coding:utf-8 -*-

"""
A package for rendering embedded Cytoscape.js object for IPython Notebook.

"""

from py2cytoscape.cytoscapejs.viewer import *

JS_LOADER_FILE = "loader.js"


def init():
    from IPython.core.display import display, Javascript

    path = os.path.abspath(os.path.dirname(__file__)) + "/" + JS_LOADER_FILE
    js_loader = open(path).read()
    return display(Javascript(js_loader))


# Load Cytoscape.js and dependent libraries
init()



import pandas as pd
import json
import requests

class Table(object):
    """The Table object provides methods for Cytoscape table manipulation"""
    def __init__(self):
        self.df = None
        self.port = None

    def push(self, target_table_name):
        BASE_URL = "http://localhost:" + self.port + "/v1/"
        HEADERS = {'Content-Type': 'application/json'}
        jsontable = json.loads(self.df.to_json(orient="records"))
        print(json.dumps(jsontable, indent=4))

    def set_port(self, port):
        self.port = str(port)

def read(filepath):
    table = Table()
    table.df = pd.read_csv(filepath)

    return table



# -*- coding:utf-8 -*-

"""
A package for converting graphs in various formats.

"""

from py2cytoscape.util.util_networkx import from_networkx, to_networkx
from py2cytoscape.util.util_graphlab import from_sgraph
from py2cytoscape.util.util_igraph import from_igraph

__all__ = ['from_networkx', 'to_networkx', 'from_igraph']


# -*- coding: utf-8 -*-
"""

Conversion utilities for igraph

"""

DEF_SCALING = 1.0


def from_igraph(igraph_network, layout=None, scale=DEF_SCALING):
    new_graph = {}
    network_data = {}
    elements = {}
    nodes = []
    edges = []

    # Convert network attributes
    network_attr = igraph_network.attributes()
    for key in network_attr:
        network_data[key] = igraph_network[key]

    # get network as a list of edges
    edges_original = igraph_network.es
    nodes_original = igraph_network.vs

    node_attr = igraph_network.vs.attributes()
    for idx, node in enumerate(nodes_original):
        new_node = {}
        data = {}
        data['id'] = str(node.index)
        data['name'] = str(node.index)
        for key in node_attr:
            data[key] = node[key]
        new_node['data'] = data
        if layout is not None:
            position = {}
            position['x'] = layout[idx][0] * scale
            position['y'] = layout[idx][1] * scale
            new_node['position'] = position

        nodes.append(new_node)

    # Add edges to the elements
    edge_attr = igraph_network.es.attributes()
    for edge in edges_original:
        new_edge = {}
        data = {}
        data['source'] = str(edge.source)
        data['target'] = str(edge.target)
        for key in edge_attr:
            data[key] = edge[key]
        new_edge['data'] = data
        edges.append(new_edge)

    elements['nodes'] = nodes
    elements['edges'] = edges
    new_graph['elements'] = elements
    new_graph['data'] = network_data

    return new_graph



import unittest
import json
import os

import igraph as ig

import networkx as nx
from py2cytoscape import util

import scipy as sp
import tempfile


# Utilities
def compare_edge_sets(nx_edges, cy_edges):
    edge_set = set()
    for cyedge in cy_edges:
        source = cyedge['data']['source']
        target = cyedge['data']['target']
        edge = (long(source), long(target))
        edge_set.add(edge)

    return edge_set.difference(nx_edges)


class NetworkConversionTests(unittest.TestCase):

    def test_networkx_emptynetwork(self):
        g = nx.Graph()
        cyjs_g = util.from_networkx(g)

        print('\n---------- Empty Test Start -----------\n')
        print(json.dumps(cyjs_g, indent=4))

        self.assertIsNotNone(cyjs_g)
        self.assertIsNotNone(cyjs_g['data'])
        self.assertEqual(0, len(cyjs_g['elements']['nodes']))
        self.assertEqual(0, len(cyjs_g['elements']['edges']))

    def test_networkx_edge_attribute(self):
        print('\n---------- Edge Att Test Start -----------\n')
        g = nx.Graph()
        g.add_edge(1, 2, interaction='itr1',  score=0.1)
        original_edge = g[1][2]
        print(original_edge.keys())
        cyjs = util.from_networkx(g)

        print(json.dumps(cyjs, indent=4))

        # There is only one edge, so this should be OK...
        edge = cyjs['elements']['edges'][0]
        print(json.dumps(edge, indent=4))

        self.assertEqual('itr1', cyjs['elements']['edges'][0]['data']['interaction'])

    def test_networkx_empty_edge_attribute(self):
        print('\n---------- Edge Att Test 2 Start -----------\n')
        g = nx.scale_free_graph(5)
        cyjs = util.from_networkx(g)

        # print(json.dumps(cyjs, indent=4))

        # There is only one edge, so this should be OK...
        edge = cyjs['elements']['edges'][0]
        print(json.dumps(edge, indent=4))
        self.assertEqual(3, len(edge['data']))

    def test_networkx_digraph_edge_attr(self):
        print('\n---------- Digraph Edge Att Test Start -----------\n')
        g = nx.DiGraph()
        g.add_path([0, 1, 2, 3, 4])
        eb = nx.edge_betweenness(g)
        nx.set_edge_attributes(g, 'eb', eb)
        cyjs = util.from_networkx(g)

        print(json.dumps(cyjs, indent=4))

        # There is only one edge, so this should be OK...
        edge = cyjs['elements']['edges'][0]
        self.assertEqual(3, len(edge['data']))

    def test_networkx_multidigraph_edge_attr(self):
        print('\n---------- Multi-Digraph Edge Att Test Start -----------\n')
        g = nx.MultiDiGraph()
        g.add_node(1)
        g.add_node(2)
        g.add_node(3)
        g.add_edge(1, 2)
        g.add_edge(1, 2, attr_dict={'foo': 'bar'})
        g.add_edge(1, 2)
        g.add_edge(1, 3)
        edges = g.edges(data=True, keys=True)
        for edge in edges:
            print(edge)

        cyjs = util.from_networkx(g)

        print(json.dumps(cyjs, indent=4))

        edge = cyjs['elements']['edges'][0]
        self.assertTrue(3 <= len(edge['data']))

    def test_networkx_ba(self):
        g = nx.barabasi_albert_graph(100, 3)
        nodes = g.nodes()
        edges = g.edges()

        g.graph['name'] = 'ba test'
        cyjs_g = util.from_networkx(g)

        print('\n---------- BA graph Test Start -----------\n')

        self.assertIsNotNone(cyjs_g)
        self.assertIsNotNone(cyjs_g['data'])
        self.assertEqual('ba test', cyjs_g['data']['name'])
        self.assertEqual(len(nodes), len(cyjs_g['elements']['nodes']))
        self.assertEqual(len(edges), len(cyjs_g['elements']['edges']))

        diff = compare_edge_sets(set(edges), cyjs_g['elements']['edges'])
        self.assertEqual(0, len(diff))

    def test_networkx_matrix(self):
        print('\n---------- Matrix Test Start -----------\n')

        g = nx.barabasi_albert_graph(30, 2)
        nodes = g.nodes()
        edges = g.edges()
        print(edges)

        mx1 = nx.adjacency_matrix(g)
        fp = tempfile.NamedTemporaryFile()
        file_name = fp.name
        sp.savetxt(file_name, mx1.toarray(), fmt='%d')

        # Load it back to matrix
        mx2 = sp.loadtxt(file_name)
        fp.close()

        g2 = nx.from_numpy_matrix(mx2)
        cyjs_g = util.from_networkx(g2)

        #print(json.dumps(cyjs_g, indent=4))

        self.assertIsNotNone(cyjs_g)
        self.assertIsNotNone(cyjs_g['data'])
        self.assertEqual(len(nodes), len(cyjs_g['elements']['nodes']))
        self.assertEqual(len(edges), len(cyjs_g['elements']['edges']))

        # Make sure all edges are reproduced
        print(set(edges))
        diff = compare_edge_sets(set(edges), cyjs_g['elements']['edges'])
        self.assertEqual(0, len(diff))

    def test_networkx_gml(self):
        g = nx.read_gml('tests/data/galFiltered.gml')
        g.graph['name'] = 'gml_test'

        cyjs_g = util.from_networkx(g)

        print('\n---------- GML Test Start -----------\n')
        # print(json.dumps(cyjs_g, indent=4))
        self.assertIsNotNone(cyjs_g)

        net_data = cyjs_g['data']
        self.assertIsNotNone(net_data)
        self.assertEqual('gml_test', net_data['name'])
        self.assertEqual(331, len(cyjs_g['elements']['nodes']))
        self.assertEqual(362, len(cyjs_g['elements']['edges']))

        nodes = cyjs_g['elements']['nodes']
        node0 = nodes[0]
        self.assertEqual(type("1"), type(node0['data']['id']))

    def test_networkx_scale_free(self):
        g = nx.scale_free_graph(100)
        edge_count = g.number_of_edges()

        g.graph['name'] = 'scale_free_test'

        cyjs_g = util.from_networkx(g)

        print('\n---------- Scale free network Test Start -----------\n')
        print('Edge count = ' + str(edge_count))
        # print(json.dumps(cyjs_g, indent=4))

        self.assertIsNotNone(cyjs_g)

        net_data = cyjs_g['data']
        self.assertIsNotNone(net_data)
        self.assertEqual('scale_free_test', net_data['name'])
        self.assertEqual(100, len(cyjs_g['elements']['nodes']))
        self.assertEqual(edge_count, len(cyjs_g['elements']['edges']))

        nodes = cyjs_g['elements']['nodes']
        node0 = nodes[0]
        self.assertEqual(type("1"), type(node0['data']['id']))

    def test_networkx_parse_network(self):
        f = open('tests/data/galFiltered.json', 'r')
        jsonData = json.load(f)

        j_nodes = jsonData['elements']['nodes']
        j_edges = jsonData['elements']['edges']

        print('\n---------- JSON Loading Test Start -----------\n')
        # print(json.dumps(jsonData, indent=4))
        g = util.to_networkx(jsonData)
        nodes = g.nodes()
        edges = g.edges()

        self.assertEqual('Yeast Network Sample', g.graph['name'])
        self.assertEqual('Sample network created by JSON export.', g.graph['description'])
        self.assertEqual(4, len(g.graph['numberList']))

        self.assertEqual(len(j_nodes), len(nodes))
        self.assertEqual(len(j_edges), len(edges))

        edge_set = set(list(map(lambda x: (long(x[0]), long(x[1])), edges)))
        self.assertEqual(0, len(compare_edge_sets(edge_set, j_edges)))

    def test_networkx_roundtrip(self):
        print('\n---------- NetworkX Data Roundtrip Test Start -----------\n')

        g = nx.newman_watts_strogatz_graph(100, 3, 0.5)
        nodes = g.nodes()
        edges = g.edges()

        # Add some attributes
        g.graph['name'] = 'original'
        g.graph['density'] = nx.density(g)

        nx.set_node_attributes(g, 'betweenness', nx.betweenness_centrality(g))
        nx.set_node_attributes(g, 'degree', nx.degree(g))
        nx.set_node_attributes(g, 'closeness', nx.closeness_centrality(g))

        nx.set_edge_attributes(g, 'eb', nx.edge_betweenness(g))

        cyjs1 = util.from_networkx(g)
        g2 = util.to_networkx(cyjs1)

        self.assertEqual(len(g2.nodes()), len(nodes))
        self.assertEqual(len(g2.edges()), len(edges))

        edge_set = set(list(map(lambda x: (long(x[0]), long(x[1])), g2.edges())))
        self.assertEqual(0, len(edge_set.difference(set(edges))))

        node_original = g.node[1]
        node_generated = g2.node['1']

        print(node_original)
        print(node_generated)

        self.assertEqual(node_original['degree'], node_generated['degree'])
        self.assertEqual(node_original['betweenness'], node_generated['betweenness'])
        self.assertEqual(node_original['closeness'], node_generated['closeness'])

    def test_from_igraph(self):
        print('---------- From igraph object to Cytoscape.js -----------\n')
        empty = ig.Graph()
        cyjs = util.from_igraph(empty)

        print(json.dumps(cyjs, indent=4))

        self.assertIsNotNone(cyjs)
        self.assertIsNotNone(cyjs['data'])
        self.assertEqual(0, len(cyjs['elements']['nodes']))
        self.assertEqual(0, len(cyjs['elements']['edges']))

    def test_from_igraph_random(self):
        print('---------- From igraph random network object to Cytoscape.js -----------\n')
        ba_graph = ig.Graph.Barabasi(100, 3)
        ba_graph['name'] = 'Barabasi'
        ba_graph['number_attr'] = 12345

        ba_graph.vs['degree'] = ba_graph.degree()
        ba_graph.vs['bw'] = ba_graph.betweenness()
        ba_graph.es['ebw'] = ba_graph.edge_betweenness()

        cyjs = util.from_igraph(ba_graph)

        # print(json.dumps(cyjs, indent=4))

        self.assertIsNotNone(cyjs)
        self.assertIsNotNone(cyjs['data'])
        self.assertEqual('Barabasi', cyjs['data']['name'])
        self.assertEqual(12345, cyjs['data']['number_attr'])
        cyjs_nodes = cyjs['elements']['nodes']
        cyjs_edges = cyjs['elements']['edges']
        self.assertEqual(len(ba_graph.vs), len(cyjs_nodes))
        self.assertEqual(len(ba_graph.es), len(cyjs_edges))

        self.assertEqual(ba_graph.vs[0]['degree'], cyjs_nodes[0]['data']['degree'])
        self.assertEqual(ba_graph.vs[0]['bw'], cyjs_nodes[0]['data']['bw'])

        # Test edge
        target_edge = ba_graph.es[10]
        edge0 = None
        for e in cyjs_edges:
            if e['data']['source'] == str(target_edge.source) \
                    and e['data']['target'] == str(target_edge.target):
                edge0 = e
                break

        self.assertIsNotNone(edge0)
        self.assertEqual(target_edge['ebw'], edge0['data']['ebw'])


if __name__ == '__main__':
    unittest.main()


def from_sgraph(sgraph):
    new_graph = {}
    elements = {}
    nodes = []
    edges = []

    nodes_original = sgraph.vertices
    el = sgraph.edges

    node_attr = nodes_original[0].keys()

    for node in nodes_original:
        new_node = {}
        data = {}
        data['id'] = node['__id']
        data['name'] = node['__id']
        for key in node_attr:
            data[key] = node[key]
        new_node['data'] = data
        nodes.append(new_node)

    for edge in el:
        new_edge = {}
        data = {}
        data['source'] = str(edge['__src_id'])
        data['target'] = str(edge['__dst_id'])
        new_edge['data'] = data
        edges.append(new_edge)

    elements['nodes'] = nodes
    elements['edges'] = edges
    new_graph['elements'] = elements

    return new_graph




"""
*******
Data conversion utility for NetworkX
*******

Convert cytoscape.js style graphs from/to NetworkX object.

https://networkx.github.io/

"""

import networkx as nx

# Special Keys
ID = 'id'
NAME = 'name'
DATA = 'data'
ELEMENTS = 'elements'

NODES = 'nodes'
EDGES = 'edges'

SOURCE = 'source'
TARGET = 'target'


def __map_table_data(columns, graph_obj):
    data = {}
    for col in columns:
        if col == 0:
            break

        data[col] = graph_obj[col]

    return data


def __create_node(node, node_id):
    new_node = {}
    node_columns = node.keys()
    data = __map_table_data(node_columns, node)
    # Override special keys
    data[ID] = str(node_id)
    data[NAME] = str(node_id)

    if 'position' in node.keys():
        position = node['position']
        new_node['position'] = position

    new_node[DATA] = data
    return new_node


def __build_multi_edge(edge_tuple, g):
    source = edge_tuple[0]
    target = edge_tuple[1]
    key = edge_tuple[2]
    data = edge_tuple[3]

    data['source'] = str(source)
    data['target'] = str(target)
    data['interaction'] = str(key)
    return {DATA: data}


def __build_edge(edge_tuple, g):
    source = edge_tuple[0]
    target = edge_tuple[1]
    data = edge_tuple[2]

    data['source'] = str(source)
    data['target'] = str(target)
    return {DATA: data}


def __build_empty_graph():
    return {
        DATA: {},
        ELEMENTS: {
            NODES: [],
            EDGES: []
        }
    }


def from_networkx(g):
    # Dictionary Object to be converted to Cytoscape.js JSON
    cygraph = __build_empty_graph()

    nodes = g.nodes()
    edge_builder = None
    if isinstance(g, nx.MultiDiGraph) or isinstance(g, nx.MultiGraph):
        edges = g.edges(data=True, keys=True)
        edge_builder = __build_multi_edge
    else:
        edges = g.edges(data=True)
        edge_builder = __build_edge

    # Map network table data
    cygraph[DATA] = __map_table_data(g.graph.keys(), g.graph)

    for node_id in nodes:
        cygraph['elements']['nodes'].append(__create_node(g.node[node_id], node_id))

    for edge in edges:
        cygraph['elements']['edges'].append(edge_builder(edge, g))

    return cygraph


def to_networkx(cyjs, directed=True):
    """
    Convert Cytoscape.js-style JSON object into NetworkX object.

    By default, data will be handles as a directed graph.
    """

    if directed:
        g = nx.MultiDiGraph()
    else:
        g = nx.MultiGraph()

    network_data = cyjs[DATA]
    if network_data is not None:
        for key in network_data.keys():
            g.graph[key] = network_data[key]

    nodes = cyjs[ELEMENTS][NODES]
    edges = cyjs[ELEMENTS][EDGES]

    for node in nodes:
        data = node[DATA]
        g.add_node(data[ID], attr_dict=data)

    for edge in edges:
        data = edge[DATA]
        source = data[SOURCE]
        target = data[TARGET]

        g.add_edge(source, target, attr_dict=data)

    return g


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Setup script for py2cytoscape

Yo install, run:

python setup.py install
"""

from setuptools import setup, find_packages

setup(
    name='py2cytoscape',
    version='0.3.3',
    description='Utility package for using Cytoscape and Cytoscape.js from Python',
    author='Keiichiro Ono',
    author_email='kono@ucsd.edu',
    url='http://www.cytoscape.org/',
    license='MIT License',
    keywords=['data visualization', 'cytoscape', 'bioinformatics'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
    ],
    test_suite='tests',
    packages=find_packages(),
    package_data={'py2cytoscape': ['cytoscapejs/*.js', 'cytoscapejs/*.html', 'cytoscapejs/*.json']}
)
